/*
 *      dxsocket.h
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */


#ifndef DXSOCKET_H
#define DXSOCKET_H

#include <QTcpSocket>
#include <QSslSocket>
#include <QTcpServer>
#include <QHostInfo>

#define BUFFER_SIZE 32768

class dxSocket: public QObject
{
    Q_OBJECT
public:
    dxSocket(bool isSSL=false);
    virtual ~dxSocket();

    void listenOn();
    void connectTo();
    qint64 read(char *buffer, int len);
    qint64 write(const char *buffer, size_t len);
    qint64 write(const QString &line);
    void startConnection();
    void setHost(const QString &address, int portD, int portH=0);
    void disconnectFrom();
    bool isConnected() const { return m_connected; }
    bool isConnecting() const { return m_connecting; }
    QString getLocalIP();
    QString getHostname(const QString &ip);
    QString getIP(const QString &hostname);
    QString getRemoteIP();
    QString stringIPToBinary(const QString &ip);
    QString binaryIPToString(const QString &ip);
    bool isRoutableIP(quint32 ipaddr);
    void setIsSSL(bool isSSL);
    QString getHost() { return m_address; }
    int getPort() { return m_portD; }
    qint64 getBytesAvailable();
    bool isIPv4(const QString &address);
signals:
    void socketConnected();
    void socketStartAccept(int);
    void socketSomeConnected();
    void socketDisconnected();
    void socketCanRead(); //new data available
    void socketWritten(qint64);
    void socketError(QString);
private:
    QTcpSocket *m_tcp, *m_client;
    QSslSocket *m_ssl;
    QTcpServer *m_server;
    bool m_connected, m_isSsl, m_connecting;
    QString m_address, m_localIP, m_remoteIP;
    int m_portD, m_portH;
    bool m_listen;
    
private slots:
    void slotConnected();
    void slotDisconnected();
    void slotCanRead();
    void slotError(QAbstractSocket::SocketError);
    void slotSomeConnected();
    void slotWritten(qint64);
    void slotSslError(const QList<QSslError> &);
};

#endif // DXSOCKET_H
