/*
 *      preferences.h
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef PREFERENCES_H
#define PREFERENCES_H

#include <QFont>
#include <QList>
#include <QMap>
#include <QKeySequence>
#include "defs.h"

#define preferences Preferences::instance()

class Preferences
{
public:
    Preferences();
    Preferences(const Preferences &p);

    //Set path for -l argv
    void setIniFile(const QString &file);
    //Get ini file
    QString getIniFile() const;
    //Save to QSettings
    void save();
    //Load from QSettings
    void load();
    static Preferences &instance();

    //General settings pane in Configdialog
    bool m_oneInstance;
    bool m_disableSortTabs;
    bool m_reconnect;
    int m_numberAttempt;
    int m_delayAttempt;
    int m_maxAway;
    bool m_closeToTray;
    bool m_useSpell;
    bool m_ownServerWindow;
    bool m_logging;
    QString m_logPath;
    bool m_autoload;
    QString m_autoloadPath;

    //Look settings pane in Configdialog
    QColor m_unreadColor;
    QColor m_highlightColor;
    int m_tabPosition;

    //Irc Text settings pane in Configdialog
    IrcColor m_colors;
    QFont m_ircFont;
    QString m_nickCompletionChar;
    bool m_sameCmd;
    bool m_sameList;
    bool m_coloredNick;
    bool m_stripColors;
    bool m_loadLog;
    int m_loadLines;

    //NickList settings pane in Configdialog
    QString m_themePath;
    QString m_themesList;
    bool m_sortUserMode;

    //Ignore settings pane in Configdialog
    QStringList m_ignoreCommandsList;
    QList<UserInfo> m_ignoreUsersList;

    //DCC settings pane in Configdialog
    QString m_dccPath;
    QString m_dccIP;
    int m_dccPortD;
    int m_dccPortH;
    int m_dccTimeout;
    bool m_autoDccChat;
    bool m_autoDccFile;

    //Notification settings pane in Configdialog
    QList<UserInfo> m_friendsList;
    bool m_sounds;
    bool m_soundConnect;
    QString m_pathConnect;
    bool m_soundDisconnect;
    QString m_pathDisconnect;
    bool m_soundMessage;
    QString m_pathMessage;
    bool m_notify;
    int m_notifyPosition;
    bool m_notifyConnect;
    bool m_notifyDisconnect;
    bool m_notifyMessage;
    bool m_notifyTray;

    //Smileys settings pane in Configdialog
    bool m_useSmileys;
    QMap<QString,QString> m_smileysMap;

    //Other settings value
    QList<ServerInfo> m_serverList;
    bool m_usersShown;
    bool m_statusShown;
    int m_x;
    int m_y;
    int m_w;
    int m_h;
    bool m_maximized;
    bool m_showSpellCombo;
    QMap<QString,QString> m_langs; //map between channel and spelllang
    QMap<QString,QKeySequence> m_keybindings;

    QKeySequence getBinding(const QString &key);
    QString getBindingName(const QString &key);
    QMap<QString,QKeySequence> getDefaultKeybindings() { return m_defaultkeybindings; }
private:
    QString m_iniFile;
    QMap<QString,QKeySequence> m_defaultkeybindings;
    QMap<QString,QString> m_namekeybindings;

    void fillDefaultKeybindings();
    void createIniPath();
    QString themeListToSeparators();
};

#endif // PREFERENCES_H
