/*
 *      irctabitem.h
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef IRCTABITEM_H
#define IRCTABITEM_H

#include <fstream>
#include "dxtabitem.h"
#include "widgets/linespell.h"
#include "engine/ircengine.h"
#include "ircitemmodel.h"
#include "ircitemsortmodel.h"

class IrcTabItem : public dxTabItem
{
    Q_OBJECT
    friend class dxirc;
public:
    IrcTabItem(const QString &name, IrcEngine *engine, TabWidget *tabbook, TYPE type=CHANNEL, int id = 0, QWidget *parent = 0);
    virtual ~IrcTabItem();

    void clearChat();
    void hideUsers();
    void showUsers();
    QString getServerName() { return m_engine ? m_engine->getServerName() : ""; }
    QString getRealServerName() { return m_engine ? m_engine->getNetworkName() : ""; }
    int getServerPort() { return m_engine ? m_engine->getServerPort() : 0; }
    QString getNickName() { return m_engine ? m_engine->getNickName() : ""; }
    void setType(const TYPE &typ, const QString &name);
    TYPE getType() { return m_type; }
    void preferencesUpdated();
    void setCommandFocus();
    void showSpellComboUpdated();
    void makeLastRowVisible();
    QString getSpellLang();
    void checkAway();
    void recreateNickIcons();
    QStringList getNicks();
    bool hasNick(const QString &nick);
    bool isSameServer(IrcEngine *engine);
    IrcEngine* getEngine() { return m_engine; }
    QString getTabName() { return m_name; }
signals:
    void highlight(TYPE);
    void newmsg(TYPE);
    void luaRequest(LuaRequest);
    void newtetris();
    void connectDialog();
    void connectServer(ServerInfo);
    void ircDccServer(IrcEngine*,QString);
    void ircDccOut(IrcEngine*,QString,QString);
    void ircDccPout(IrcEngine*,QString,QString);
    void ircPrivmsg(IrcEngine*, QString nick, QString to, QString msg);
    void ircAction(IrcEngine*, QString nick, QString to, QString msg);
    void ircQuery(IrcEngine*, QString with);
    void ownMsg(QString);
    void command(QString);
    void addIgnoreCommand(QString);
    void removeIgnoreCommand(QString);
    void addIgnoreUser(QString);
    void removeIgnoreUser(QString);
    void ircQuit();
    void newlog(QString logpath);
    void newname(QString);
private:
    IrcEngine *m_engine;
    TYPE m_type;
    QSplitter *m_splitter;
    QListView *m_users;
    QTextBrowser *m_text;
    LineSpell *m_commandline;
    QLineEdit *m_topicline;
    int m_currentPosition, m_historyMax, m_numberUsers, m_pics;
    bool m_iamOp;
    bool m_editableTopic;
    bool m_sendPipe;
    bool m_scriptHasAll, m_scriptHasOwnMsg;
    bool m_useNickIcon;
    bool m_loadLog;
    QList<QString> m_commandsHistory, m_pipeStrings;
    QString m_topic;
    int m_maxLen;
    std::ofstream *m_logstream;
    QComboBox *m_spellLangs;
    QTimer *m_pipeTimer;
    QTimer *m_eggTimer;
    QProcess *m_pipe;
    QString m_name;
    IrcItemModel m_model;
    IrcItemSortModel m_sortModel;

    bool isNoCurrent();
    void appendIrcText(QString msg, QString time, bool disableStrip=false, bool logLine=true);
    void appendIrcStyledText(QString styled, int stylenum, QString time, bool disableStrip=false, bool logLine=true);
    void appendIrcNickText(QString nick, QString msg, QString time, bool logLine=true);
    void appendMyMsg(QString msg, QString time);
    void startLogging();
    void stopLogging();
    void logLine(const QString &line, const QString &time);
    bool highlightNeeded(const QString &msg);
    bool isRightForServerMsg();
    bool isCommandIgnored(const QString &command);
    void addUser(const QString &user, UserMode mode);
    void removeUser(const QString &user);
    void changeNickUser(const QString &nick, const QString &newnick);
    void setUserMode(const QString &nick, UserMode mode);
    UserMode getUserMode(const char mode);
    void onBan(const QString &banmask, const bool &sign, const QString &sender, const QString &time);
    void onAway();
    bool isChannel(const QString &text);
    QStringList cutText(QString, int);
    void loadLogLines();
    void listLogFiles();
    void completeHelpCommand();
    bool processCommand(const QString &);
    bool showHelp(QString command);
    void processCompletion();
    bool commandScript(const QString &commandtext);
    bool commandAdmin(const QString &commandtext);
    bool commandAway(const QString &commandtext);
    bool commandBanlist(const QString &commandtext);
    bool commandBoats(const QString &commandtext);
    bool commandConnect(const QString &commandtext);
    bool commandCommands();
    bool commandCtcp(const QString &commandtext);
    bool commandCycle(const QString &commandtext);
    bool commandDcc(const QString &commandtext);
    bool commandDeop(const QString &commandtext);
    bool commandDevoice(const QString &commandtext);
    bool commandDisconnect(const QString &commandtext);
    bool commandDxirc();
    bool commandEgg();
    bool commandExec(const QString &commandtext);
    bool commandHelp(const QString &commandtext);
    bool commandIgnore(const QString &commandtext);
    bool commandInvite(const QString &commandtext);
    bool commandJoin(const QString &commandtext);
    bool commandKick(const QString &commandtext);
    bool commandKill(const QString &commandtext);
    bool commandList(const QString &commandtext);
    bool commandLog(const QString &commandtext);
    bool commandLua(const QString &commandtext);
    bool commandMe(const QString &commandtext);
    bool commandMode(const QString &commandtext);
    bool commandMsg(const QString &commandtext);
    bool commandNames(const QString &commandtext);
    bool commandNick(const QString &commandtext);
    bool commandNotice(const QString &commandtext);
    bool commandOp(const QString &commandtext);
    bool commandOper(const QString &commandtext);
    bool commandPart(const QString &commandtext);
    bool commandQuery(const QString &commandtext);
    bool commandQuit();
    bool commandQuote(const QString &commandtext);
    bool commandSay(const QString &commandtext);
    bool commandStats(const QString &commandtext);
    bool commandTetris();
    bool commandTime();
    bool commandTopic(const QString &commandtext);
    bool commandVoice(const QString &commandtext);
    bool commandWallops(const QString &commandtext);
    bool commandWho(const QString &commandtext);
    bool commandWhoami();
    bool commandWhois(const QString &commandtext);
    bool commandWhowas(const QString &commandtext);
    bool notConnected();
private slots:
    void slotPopup(const QPoint &pos);
    void slotProcessCommandline();
    void slotProcessTopicline();
    void slotOnPipe();
    void slotPipeTimeout();
    void slotEggTimeout();
    void slotIrcConnect(QString msg);
    void slotIrcDisconnect(QString msg); //send at closeconnection w/o reconnection
    void slotIrcReconnect(QString msg); //send at closeconnection with reconnection
    void slotIrcReconnectAll(QString msg); //send at menu "Reconnect all"
    void slotIrcError(QString msg);
    void slotIrcPrivmsg(QString nick, QString to, QString msg);
    void slotIrcAction(QString nick, QString to, QString msg);
    void slotIrcCtcpReply(QString nick, QString msg);
    void slotIrcCtcpRequest(QString nick, QString request);
    void slotIrcJoin(QString nick, QString channel);
    void slotIrcPart(QString nick, QString channel, QString reason);
    void slotIrcQuit(QString nick, QString reason);
    void slotIrcTopic(QString nick, QString channel, QString topic);
    void slotIrcNick(QString nick, QString newnick);
    void slotIrcMode(QString mode, QString who);
    void slotIrcUmode(QString moderator, QString channel, QString modes, QString args);
    void slotIrcChmode(QString channel, QString modes);
    void slotIrcServerReply(QString msg); //command 001,002,....
    void slotIrcServerError(QString msg); //command 400~599
    void slotIrc301(QString nick, QString msg); //RPL_AWAY
    void slotIrc305(QString msg); //RPL_UNAWAY
    void slotIrc306(QString msg); //RPL_NOWAWAY
    void slotIrc331(QString channel, QString msg); //RPL_NOTOPIC
    void slotIrc332(QString channel, QString msg); //RPL_TOPIC
    void slotIrc333(QString channel, QString msg); //RPL_TOPICSETBY
    void slotIrc353(QString channel, QString nicks, bool list); //RPL_NAMREPLY
    void slotIrc366(QString channel, QString msg, bool list); //RPL_ENDOFNAMES
    void slotIrc372(QString msg); //RPL_MOTD
    void slotIrcKick(QString nick, QString to, QString channel, QString reason);
    void slotIrcNotice(QString to, QString msg);
    void slotIrcChnotice(QString nick, QString to, QString msg);
    void slotIrcAway(QString who);
    void slotIrcEndMotd();
    void slotIrcNickInfo(NickInfo info);
    void slotPopupQuery();
    void slotPopupWhois();
    void slotPopupDccChat();
    void slotPopupSendFile();
    void slotPopupIgnore();
    void slotPopupOp();
    void slotPopupDeop();
    void slotPopupVoice();
    void slotPopupDevoice();
    void slotPopupKick();
    void slotPopupBan();
    void slotPopupKickBan();
    void slotPopupDeAway();
    void slotPopupAway();
    void slotDoubleClickUsers(QModelIndex);
protected:
    bool eventFilter(QObject *obj, QEvent *ev);
    void appendText(QString msg, bool highlight, bool logLine=true);
    void appendStyledText(QString text, int style, bool highlight, bool disableStrip=false, bool logLine=true);
    bool processLine(const QString &);
    void setHasAllCommand(bool);
    void setHasOwnMsg(bool);
    void showLog(const QString &path);
};

#endif  /* IRCTABITEM_H */

